# simple DB API testcase

import numpy
import pandas


class TestSimpleDBAPI(object):
    def test_regular_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchall()
        assert result == [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [None]], "Incorrect result returned"

    def test_numpy_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchnumpy()
        arr = numpy.ma.masked_array(numpy.arange(11))
        arr.mask = [False] * 10 + [True]
        numpy.testing.assert_array_equal(result['i'], arr, "Incorrect result returned")

        duckdb_cursor.execute('SELECT * FROM timestamps')
        result = duckdb_cursor.fetchnumpy()
        arr = numpy.array(['1992-10-03 18:34:45', '2010-01-01 00:00:01', None], dtype="datetime64[ms]")
        arr = numpy.ma.masked_array(arr)
        arr.mask = [False, False, True]
        numpy.testing.assert_array_equal(result['t'], arr, "Incorrect result returned")

    def test_pandas_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchdf()
        arr = numpy.ma.masked_array(numpy.arange(11))
        arr.mask = [False] * 10 + [True]
        arr = {'i': arr}
        arr = pandas.DataFrame.from_dict(arr)
        # assert str(result) == str(arr), "Incorrect result returned"
        pandas.testing.assert_frame_equal(result, arr)

        duckdb_cursor.execute('SELECT * FROM timestamps')
        result = duckdb_cursor.fetchdf()
        df = pandas.DataFrame({
            't': pandas.to_datetime(['1992-10-03 18:34:45', '2010-01-01 00:00:01', None])
        })
        pandas.testing.assert_frame_equal(result, df)

    # def test_numpy_creation(self, duckdb_cursor):
    #     # numpyarray = {'i': numpy.arange(10), 'v': numpy.random.randint(100, size=(1, 10))}  # segfaults
    #     data_dict = {'i': numpy.arange(10), 'v': numpy.random.randint(100, size=10)}
    #     duckdb_cursor.create('numpy_creation', data_dict)
    #     duckdb_cursor.commit()

    #     duckdb_cursor.execute('SELECT * FROM numpy_creation')
    #     result = duckdb_cursor.fetchnumpy()

    #     numpy.testing.assert_array_equal(result['i'], data_dict['i'])
    #     numpy.testing.assert_array_equal(result['v'], data_dict['v'])

    # def test_pandas_creation(self, duckdb_cursor):
    #     data_dict = {'i': numpy.arange(10), 'v': numpy.random.randint(100, size=10)}
    #     dframe = pandas.DataFrame.from_dict(data_dict)
    #     duckdb_cursor.create('dframe_creation', dframe)

    #     duckdb_cursor.execute('SELECT * FROM dframe_creation')
    #     result = duckdb_cursor.fetchnumpy()

    #     numpy.testing.assert_array_equal(result['i'], data_dict['i'])
    #     numpy.testing.assert_array_equal(result['v'], data_dict['v'])

    # def test_numpy_insertion(self, duckdb_cursor):
    #     data_dict = {'i': numpy.arange(10), 'v': numpy.random.randint(100, size=10)}
    #     duckdb_cursor.execute("CREATE TABLE numpy_insertion (i INT, v INT)")
    #     duckdb_cursor.insert('numpy_insertion', data_dict)
    #     duckdb_cursor.commit()

    #     duckdb_cursor.execute("SELECT * FROM numpy_insertion")
    #     result = duckdb_cursor.fetchnumpy()

    #     numpy.testing.assert_array_equal(result['i'], data_dict['i'])
    #     numpy.testing.assert_array_equal(result['v'], data_dict['v'])

    # def test_pandas_insertion(self, duckdb_cursor):
    #     data_dict = {'i': numpy.arange(10), 'v': numpy.random.randint(100, size=10)}
    #     dframe = pandas.DataFrame.from_dict(data_dict)
    #     duckdb_cursor.execute("CREATE TABLE pandas_insertion (i INT, v INT)")
    #     duckdb_cursor.insert('pandas_insertion', dframe)
    #     duckdb_cursor.commit()

    #     duckdb_cursor.execute("SELECT * FROM pandas_insertion")
    #     result = duckdb_cursor.fetchnumpy()

    #     numpy.testing.assert_array_equal(result['i'], data_dict['i'])
    #     numpy.testing.assert_array_equal(result['v'], data_dict['v'])

    # def test_masked_array_insertion(self, duckdb_cursor):
    #     data_dict = {'i': numpy.ma.masked_array(numpy.arange(10), mask=([False]*9 + [True]))}
    #     duckdb_cursor.execute("CREATE TABLE masked_array_insertion (i INT)")
    #     duckdb_cursor.insert("masked_array_insertion", data_dict)
    #     duckdb_cursor.commit()

    #     duckdb_cursor.execute("SELECT * FROM masked_array_insertion")
    #     result = duckdb_cursor.fetchnumpy()

    #     numpy.testing.assert_array_equal(result['i'], data_dict['i'])



import numpy
import os
import pytest
import shutil

import duckdb

@pytest.fixture(scope="function")
def duckdb_empty_cursor(request):
    test_dbfarm = tmp_path.resolve().as_posix()

    connection = duckdb.connect('')
    cursor = connection.cursor()
    return cursor

@pytest.fixture(scope="function")
def duckdb_cursor(request):

    connection = duckdb.connect('')
    cursor = connection.cursor()
   # cursor.create('integers', {'i': numpy.arange(10)})
    cursor.execute('CREATE TABLE integers (i integer)')
    cursor.execute('INSERT INTO integers VALUES (0),(1),(2),(3),(4),(5),(6),(7),(8),(9),(NULL)')
    cursor.execute('CREATE TABLE timestamps (t timestamp)')
    cursor.execute("INSERT INTO timestamps VALUES ('1992-10-03 18:34:45'), ('2010-01-01 00:00:01'), (NULL)")
    return cursor


@pytest.fixture(scope="function")
def duckdb_cursor_autocommit(request, tmp_path):
    test_dbfarm = tmp_path.resolve().as_posix()

    def finalizer():
        duckdb.shutdown()
        if tmp_path.is_dir():
            shutil.rmtree(test_dbfarm)

    request.addfinalizer(finalizer)

    connection = duckdb.connect(test_dbfarm)
    connection.set_autocommit(True)
    cursor = connection.cursor()
    return (cursor, connection, test_dbfarm)


@pytest.fixture(scope="function")
def initialize_duckdb(request, tmp_path):
    test_dbfarm = tmp_path.resolve().as_posix()

    def finalizer():
        duckdb.shutdown()
        if tmp_path.is_dir():
            shutil.rmtree(test_dbfarm)

    request.addfinalizer(finalizer)

    duckdb.connect(test_dbfarm)
    return test_dbfarm



import json
import os

MAX_NODES = 1000

raphael_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'raphael.js')
treant_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'treant.js')


def generate_html(parsed_json):
	global current_tone, execution_time, total_nodes
	# tones for the different nodes, we use "earth tones"
	tones = [
		"#493829",
		"#816C5B",
		"#A9A18C",
		"#613318",
		"#B99C6B",
		"#A15038",
		"#8F3B1B",
		"#D57500",
		"#DBCA69",
		"#404F24",
		"#668D3C",
		"#BDD09F",
		"#4E6172",
		"#83929F",
		"#A3ADB8"
	]
	# set fixed tones for the various common operators
	fixed_tone_map = {
		"ORDER_BY": 8,
		"HASH_GROUP_BY": 7,
		"FILTER": 13,
		"SEQ_SCAN": 4,
		"HASH_JOIN": 10,
		"PROJECTION": 5,
		"LIMIT": 0,
		"PIECEWISE_MERGE_JOIN": 1,
		"DISTINCT": 11,
		"DELIM_SCAN": 9
	}
	# remaining (unknown) operators just fetch tones in order
	remaining_tones = []
	for i in range(len(tones)):
		if i not in fixed_tone_map.values():
			remaining_tones.append(i)

	current_tone = 0
	# first parse the total time
	total_time = float(parsed_json["result"])

	# assign an impact factor to nodes
	# this is used for rendering opacity
	execution_time = 0
	node_timings = {}
	node_list = []
	def gather_nodes(node):
		global execution_time
		node_timing = float(node["timing"])
		if node['name'] not in node_timings:
			node_timings[node['name']] = 0
		node_timings[node['name']] += node_timing
		execution_time += node_timing
		node_list.append((node_timing, node))
		children = node["children"]
		for i in range(len(children)):
			gather_nodes(children[i])

	def timing_to_str(timing):
		timing_str = "%.3f" % (timing,)
		if timing_str == "0.000":
			return "0.0"
		# if "e-" in timing_str:
		# 	timing_str = "0.0"
		return timing_str

	def to_percentage(total_time, time):
		fraction = 100 * (time / total_time)
		if fraction < 0.1:
			return "0%"
		else:
			return "%.1f" % (fraction,) + "%"


	# first gather all nodes
	gather_nodes(parsed_json["tree"])

	# get the timing of the different query phases
	meta_info = ""
	if "timings" in parsed_json:
		keys = []
		timings = {}
		for entry in parsed_json["timings"]:
			timings[entry] = parsed_json["timings"][entry]
			keys.append(entry)
		keys.sort()
		meta_info = """<table class="meta-info">
<tr class="phaseheader metainfo">
<th style="font-weight:bold;">Phase</th>
<th style="font-weight:bold;">Time</th>
<th style="font-weight:bold;">Percentage</th>
</tr>
"""
		# add the total time
		meta_info += """<tr class="metainfo mainphase"><th style="font-weight:bold;">Total Time</th><th style="font-weight:bold;">${TOTAL_TIME}s</th><th></th></tr>""".replace("${TOTAL_TIME}", timing_to_str(total_time))
		# add the execution phase
		meta_info += """<tr class="metainfo mainphase"><th>Execution</th><th>${EXECUTION_TIME}s</th><th>${PERCENTAGE}</th></tr>""".replace("${EXECUTION_TIME}", str(execution_time)).replace("${PERCENTAGE}", to_percentage(total_time, execution_time))
		execution_nodes = node_timings.keys()
		execution_nodes = sorted(execution_nodes, key=lambda x: -node_timings[x])
		for node in execution_nodes:
			meta_info += '<tr class="metainfo subphase"><th>${NODE}</th><th>${NODE_TIME}s</th><th>${PERCENTAGE}</th></tr>'.replace("${NODE}", node).replace("${NODE_TIME}", timing_to_str(node_timings[node])).replace("${PERCENTAGE}", to_percentage(total_time, node_timings[node]))


		# add the planning/optimizer phase to the table
		for key in keys:
			row_class = 'subphase' if ">" in key else 'mainphase'
			meta_info += '<tr class="metainfo %s">' % (row_class)
			meta_info += '<th>'
			if ">" in key:
				# subphase
				meta_info += "> " + key.split(" > ")[-1].title().replace("_", " ")
			else:
				# primary phase
				meta_info += key.title().replace("_", " ")
			meta_info += "</th>"
			meta_info += '<th>' + timing_to_str(timings[key]) + "s" + "</th>"
			meta_info += "<th>" + to_percentage(total_time, timings[key]) + "</th>"
			meta_info += "</tr>"
		meta_info += '</table>'

	# assign impacts based on % of execution spend in the node
	# <0.1% = 0
	# <1% = 1
	# <5% = 2
	# anything else: 3
	for i in range(len(node_list)):
		percentage = 100 * (node_list[i][0] / total_time)
		if percentage < 0.1:
			node_list[i][1]["impact"] = 0
		elif percentage < 1:
			node_list[i][1]["impact"] = 1
		elif percentage < 5:
			node_list[i][1]["impact"] = 2
		else:
			node_list[i][1]["impact"] = 3


	# now convert the JSON to the format used by treant.js
	def add_spaces_to_big_number(number):
		if len(number) > 4:
			return add_spaces_to_big_number(number[:-3]) + " " + number[-3:]
		else:
			return number

	total_nodes = 0
	def convert_json(node):
		global total_nodes
		total_nodes += 1
		# format is { text { name, title, contact }, HTMLclass, children: [...] }
		global current_tone
		result = '{ "text": {'
		result += '"name": "' + node["name"] + ' (' + str(node["timing"]) + 's)",\n'
		title = ""
		splits = node["extra_info"].split('\n')
		for i in range(len(splits)):
			if len(splits[i]) > 0:
				title += splits[i].replace('"', '') + "<br>"
		result += '"title": "' + title + '",\n'
		result += '"contact": "[' + add_spaces_to_big_number(str(node["cardinality"])) + ']"},\n'
		if node["name"] not in fixed_tone_map:
			fixed_tone_map[node["name"]] = remaining_tones[current_tone]
			current_tone += 1
			if current_tone >= len(remaining_tones):
				current_tone = 0
		if node["impact"] == 0:
			impact_class = "small-impact"
		elif node["impact"] == 1:
			impact_class = "medium-impact"
		elif node["impact"] == 2:
			impact_class = "high-impact"
		else:
			impact_class = "most-impact"
		result += '"HTMLclass": "' + impact_class + ' tone-' + str(fixed_tone_map[node["name"]]) + '",\n'
		result += '"children": ['
		children = node["children"]
		for i in range(len(children)):
			result += convert_json(children[i])
			if total_nodes > MAX_NODES:
				break
			if  i + 1 < len(children):
				result += ","
		result += "]}\n"
		return result

	new_json = convert_json(parsed_json["tree"])

	# create the chart info object that is used to create the final tree
	chart_info = """
	var chart_config = {
		chart: {
			container: "#query-profile",
			nodeAlign: "BOTTOM",
			connectors: {
				type: 'step'
			},
			node: {
				HTMLclass: 'tree-node'
			}
		},
		nodeStructure: ${JSON_STRUCTURE}
	}
	""".replace("${JSON_STRUCTURE}", new_json)

	# generate tone CSS from the tones we set above
	tone_text = ""
	for i in range(len(tones)):
		tone_text += """
		.tone-${INDEX} {
			background-color: ${COLOR};
		}""".replace("${INDEX}", str(i)).replace("${COLOR}", tones[i])

	css = """<style>
	.Treant { position: relative; overflow: hidden; padding: 0 !important; }
	.Treant > .node,
	.Treant > .pseudo { position: absolute; display: block; visibility: hidden; }
	.Treant.loaded .node,
	.Treant.loaded .pseudo { visibility: visible; }
	.Treant > .pseudo { width: 0; height: 0; border: none; padding: 0; }
	.Treant .collapse-switch { width: 3px; height: 3px; display: block; border: 1px solid black; position: absolute; top: 1px; right: 1px; cursor: pointer; }
	.Treant .collapsed .collapse-switch { background-color: #868DEE; }
	.Treant > .node img {	border: none; float: left; }

	body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,form,fieldset,input,textarea,p,blockquote,th,td { margin:0; padding:0; }
	table { border-collapse:collapse; border-spacing:0; }
	fieldset,img { border:0; }
	address,caption,cite,code,dfn,em,strong,th,var { font-style:normal; font-weight:normal; }
	caption,th { text-align:left; }
	h1,h2,h3,h4,h5,h6 { font-size:100%; font-weight:normal; }
	q:before,q:after { content:''; }
	abbr,acronym { border:0; }

	body { background: #fff; }
	/* optional Container STYLES */
	.chart { height: 500px; margin: 5px; width: 100%;  }
	.Treant > .node {  }
	.Treant > p { font-family: "HelveticaNeue-Light", "Helvetica Neue Light", "Helvetica Neue", Helvetica, Arial, "Lucida Grande", sans-serif; font-weight: bold; font-size: 12px; }

	.tree-node {
		padding: 2px;
		-webkit-border-radius: 3px;
		-moz-border-radius: 3px;
		border-radius: 3px;
		background-color: #ffffff;
		border: 1px solid #000;
		width: 200px;
		font-family: Tahoma;
		font-size: 12px;
	}

	.metainfo {
		font-family: Tahoma;
		font-weight: bold;
	}

	.metainfo > th {
		padding: 1px 5px;
	}

	.mainphase {
		background-color: #668D3C;
	}
	.subphase {
		background-color: #B99C6B;
	}
	.phaseheader {
		background-color: #83929F;
	}

	.node-name {
		font-weight: bold;
		text-align: center;
		font-size: 14px;
	}

	.node-title {
		text-align: center;
	}

	.node-contact {
		font-weight: bold;
		text-align: center;
	}

	.small-impact {
		opacity: 0.7;
	}

	.medium-impact {
		opacity: 0.8;
	}

	.high-impact {
		opacity: 0.9;
	}

	.most-impact {
		opacity: 1;
	}


	${TONES}
		</style>""".replace("${TONES}", tone_text)
	chart_script = """
		<script>
		${CHART_INFO}

			// create a dummy chart to figure out how wide/high it is
			new Treant(chart_config );
			max_height = 0
			document.querySelectorAll('.tree-node').forEach(function(node) {
				top_px = parseInt(node.style.top.substring(0, node.style.top.length - 2))
				if (top_px > max_height) {
					max_height = top_px
				}
			});
			// now remove the chart we just created
			document.querySelectorAll('#query-profile').forEach(function(node) {
				node.innerHTML = ""
			});
			// resize the chart
			document.querySelectorAll('.chart').forEach(function(node) {
				node.style.height = (max_height + 200) + "px"
			});
			// and recreate it
			new Treant(chart_config );


		</script>""".replace("${CHART_INFO}", chart_info)
	libraries = """
	<script>
	${RAPHAEL}
	</script>
	<script>
	${TREANT}
	</script>
	""".replace("${RAPHAEL}", open(raphael_path).read()).replace("${TREANT}", open(treant_path).read())
	return {
		'css': css,
		'meta_info': meta_info,
		'chart_script': chart_script,
		'libraries': libraries
	}

def generate_ipython(json_input):
	from IPython.core.display import HTML

	parsed_json = json.loads(json_input)

	html_output = generate_html(parsed_json)

	return HTML("""
	${CSS}
	${LIBRARIES}
	<div class="chart" id="query-profile"></div>
	${CHART_SCRIPT}
	""".replace("${CSS}", html_output['css']).replace('${CHART_SCRIPT}', html_output['chart_script']).replace('${LIBRARIES}', html_output['libraries']))


def generate(input_file, output_file):
	with open(input_file) as f:
		text = f.read()
		# we only render the first tree, extract it
		text = '{ "result"' + text.split('{ "result"')[1]

	parsed_json = json.loads(text)

	html_output = generate_html(parsed_json)

	# finally create and write the html
	with open(output_file, "w+") as f:
		f.write("""
	<!DOCTYPE html>
	<html>
		<head>
		<meta charset="utf-8">
		<meta name="viewport" content="width=device-width">
		<title>Query Profile Graph for Query</title>
		${CSS}
	</head>
	<body>
		${LIBRARIES}

		<div id="meta-info">${META_INFO}</div>
		<div class="chart" id="query-profile"></div>

		${CHART_SCRIPT}
	</body>
	</html>
		""".replace("${CSS}", html_output['css']).replace("${META_INFO}", html_output['meta_info']).replace('${CHART_SCRIPT}', html_output['chart_script']).replace('${LIBRARIES}', html_output['libraries']))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import numpy
import sys
import subprocess
import platform
import shutil

import distutils.spawn
from setuptools import setup, Extension
from setuptools.command.sdist import sdist
from distutils.command.build_ext import build_ext


# make sure we are in the right directory
os.chdir(os.path.dirname(os.path.realpath(__file__)))

ARCHIVE_EXT = 'a'
LIB_PREFIX = 'lib'
if os.name == 'nt':
    ARCHIVE_EXT = 'lib'
    LIB_PREFIX = 'Release/'

DIR_PREFIX = 'src/duckdb'
if not os.path.exists(DIR_PREFIX):
     # this is a build from within the tools/pythonpkg directory
    DIR_PREFIX = '../../'

def get_library_name(lib):
    return LIB_PREFIX + lib + '.' + ARCHIVE_EXT

DEFAULT_BUILD_DIR = os.path.join(DIR_PREFIX, 'build', 'release_notest')

BUILD_DIR = DEFAULT_BUILD_DIR
if 'DUCKDB_PYTHON_TARGET' in os.environ:
    BUILD_DIR = os.environ['DUCKDB_PYTHON_TARGET']


INCLUDE_DIR = os.path.join(DIR_PREFIX,  'src', 'include')

DUCKDB_LIB = os.path.join(BUILD_DIR, 'src', get_library_name('duckdb_static'))
PG_LIB = os.path.join(BUILD_DIR, 'third_party', 'libpg_query', get_library_name('pg_query'))
RE2_LIB = os.path.join(BUILD_DIR, 'third_party', 're2', get_library_name('re2'))
MINIZ_LIB = os.path.join(BUILD_DIR, 'third_party', 'miniz', get_library_name('miniz'))


# wrapper that builds the main DuckDB library first
class CustomBuiltExtCommand(build_ext):
    def build_duckdb(self):
        cmake_bin = distutils.spawn.find_executable('cmake')
        if (cmake_bin is None):
            raise Exception('DuckDB needs cmake to build from source')

        wd = os.getcwd()
        os.chdir(DIR_PREFIX)
        if not os.path.exists('build/release_notest'):
            os.makedirs('build/release_notest')
        os.chdir('build/release_notest')

        configcmd = 'cmake -DCMAKE_BUILD_TYPE=Release -DLEAN=1 ../..'
        buildcmd = 'cmake --build . --target duckdb_static'

        if os.name == 'nt':
            if platform.architecture()[0] == '64bit':
                configcmd += ' -DCMAKE_GENERATOR_PLATFORM=x64'
            buildcmd += ' --config Release'

        subprocess.Popen(configcmd.split(' ')).wait()
        subprocess.Popen(buildcmd.split(' ')).wait()

        os.chdir(wd)

    def run(self):
        if BUILD_DIR == DEFAULT_BUILD_DIR:
            self.build_duckdb()

        for library in [DUCKDB_LIB, PG_LIB, RE2_LIB, MINIZ_LIB]:
            if not os.path.isfile(library):
                raise Exception('Build failed: could not find required library file "%s"' % library)
        print(INCLUDE_DIR)
        build_ext.run(self)

# create a distributable directory structure
class CustomSdistCommand(sdist):
    def run(self):
        if os.path.exists('src/duckdb'):
            shutil.rmtree('src/duckdb')
        if not os.path.exists('src/duckdb/third_party'):
            os.makedirs('src/duckdb/third_party')
        shutil.copyfile('../../CMakeLists.txt', 'src/duckdb/CMakeLists.txt')
        shutil.copyfile('../../third_party/CMakeLists.txt', 'src/duckdb/third_party/CMakeLists.txt')
        shutil.copytree('../../src', 'src/duckdb/src')
        shutil.copytree('../../third_party/libpg_query', 'src/duckdb/third_party/libpg_query')
        shutil.copytree('../../third_party/hyperloglog', 'src/duckdb/third_party/hyperloglog')
        shutil.copytree('../../third_party/re2', 'src/duckdb/third_party/re2')
        shutil.copytree('../../third_party/miniz', 'src/duckdb/third_party/miniz')
        sdist.run(self)

includes = [numpy.get_include(), INCLUDE_DIR, '.']
sources = ['connection.cpp', 'cursor.cpp', 'module.cpp']

toolchain_args = ['-std=c++11', '-Wall']
if platform.system() == 'Darwin':
    toolchain_args.extend(['-stdlib=libc++', '-mmacosx-version-min=10.7'])

libduckdb = Extension('duckdb',
    include_dirs=includes,
    sources=sources,
    extra_compile_args=toolchain_args,
    extra_link_args=toolchain_args,
    language='c++',
    extra_objects=[DUCKDB_LIB, PG_LIB, RE2_LIB, MINIZ_LIB])

# Only include pytest-runner in setup_requires if we're invoking tests
if {'pytest', 'test', 'ptr'}.intersection(sys.argv):
    setup_requires = ['pytest-runner']
else:
    setup_requires = []

setup(
    name = "duckdb",
    description = 'DuckDB embedded database',
    keywords = 'DuckDB Database SQL OLAP',
    url="https://github.com/cwida/duckdb",
    long_description = '',
    install_requires=[ # these versions are still available for Python 2, newer ones aren't
         'numpy>=1.14', 
         'pandas>=0.23'
    ],
    packages=['duckdb_query_graph'],
    include_package_data=True,
    setup_requires=setup_requires + ["setuptools_scm"],
    use_scm_version = {"root": "../..", "relative_to": __file__},
    tests_require=['pytest'],
    classifiers = [
        'Topic :: Database :: Database Engines/Servers',
        'Intended Audience :: Developers',
        'Development Status :: 3 - Alpha'
    ],
    cmdclass={
       'build_ext': CustomBuiltExtCommand,
       'sdist': CustomSdistCommand
    },
    ext_modules = [libduckdb],
    maintainer = "Hannes Muehleisen",
    maintainer_email = "hannes@cwi.nl"
)



# multiple result sets

import numpy


class TestMultipleResultSets(object):
    def test_regular_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchall()
        assert result == [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [None]], "Incorrect result returned"

    def test_numpy_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchnumpy()
        expected = numpy.ma.masked_array(numpy.arange(11), mask=([False]*10 + [True]))

        numpy.testing.assert_array_equal(result['i'], expected)



#simple DB API testcase


class TestSimpleDBAPI(object):
    def test_regular_selection(self, duckdb_cursor):
        duckdb_cursor.execute('SELECT * FROM integers')
        result = duckdb_cursor.fetchall()
        assert result == [[0],[1],[2],[3],[4],[5],[6],[7],[8],[9], [None]], "Incorrect result returned"
